library angular2.src.forms.directives.checkbox_value_accessor;

import "package:angular2/angular2.dart" show Directive, Renderer, ElementRef;
import "ng_control.dart" show NgControl;
import "control_value_accessor.dart" show ControlValueAccessor;
import "shared.dart" show setProperty;

/**
 * The accessor for writing a value and listening to changes on a checkbox input element.
 *
 *  # Example
 *  ```
 *  <input type="checkbox" [ng-control]="rememberLogin">
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "input[type=checkbox][ng-control],input[type=checkbox][ng-form-control],input[type=checkbox][ng-model]",
    host: const {
  "(change)": "onChange(\$event.target.checked)",
  "(blur)": "onTouched()",
  "[checked]": "checked",
  "[class.ng-untouched]": "cd.control?.untouched == true",
  "[class.ng-touched]": "cd.control?.touched == true",
  "[class.ng-pristine]": "cd.control?.pristine == true",
  "[class.ng-dirty]": "cd.control?.dirty == true",
  "[class.ng-valid]": "cd.control?.valid == true",
  "[class.ng-invalid]": "cd.control?.valid == false"
})
class CheckboxControlValueAccessor implements ControlValueAccessor {
  NgControl cd;
  Renderer renderer;
  ElementRef elementRef;
  bool checked;
  Function onChange;
  Function onTouched;
  CheckboxControlValueAccessor(this.cd, this.renderer, this.elementRef) {
    this.onChange = (_) {};
    this.onTouched = (_) {};
    cd.valueAccessor = this;
  }
  writeValue(value) {
    // both this.checked and setProperty are required at the moment

    // remove when a proper imperative API is provided
    this.checked = value;
    setProperty(this.renderer, this.elementRef, "checked", value);
  }
  void registerOnChange(fn) {
    this.onChange = fn;
  }
  void registerOnTouched(fn) {
    this.onTouched = fn;
  }
}
