library angular2.src.forms.directives.default_value_accessor;

import "package:angular2/angular2.dart" show Directive, Renderer, ElementRef;
import "ng_control.dart" show NgControl;
import "control_value_accessor.dart" show ControlValueAccessor;
import "package:angular2/src/facade/lang.dart" show isBlank;
import "shared.dart" show setProperty;

/**
 * The default accessor for writing a value and listening to changes that is used by the
 * {@link NgModel}, {@link NgFormControl}, and {@link NgControlName} directives.
 *
 *  # Example
 *  ```
 *  <input type="text" [(ng-model)]="searchQuery">
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "input:not([type=checkbox])[ng-control],textarea[ng-control],input:not([type=checkbox])[ng-form-control],textarea[ng-form-control],input:not([type=checkbox])[ng-model],textarea[ng-model]",
    host: const {
  "(change)": "onChange(\$event.target.value)",
  "(input)": "onChange(\$event.target.value)",
  "(blur)": "onTouched()",
  "[value]": "value",
  "[class.ng-untouched]": "cd.control?.untouched == true",
  "[class.ng-touched]": "cd.control?.touched == true",
  "[class.ng-pristine]": "cd.control?.pristine == true",
  "[class.ng-dirty]": "cd.control?.dirty == true",
  "[class.ng-valid]": "cd.control?.valid == true",
  "[class.ng-invalid]": "cd.control?.valid == false"
})
class DefaultValueAccessor implements ControlValueAccessor {
  NgControl cd;
  Renderer renderer;
  ElementRef elementRef;
  String value = null;
  Function onChange;
  Function onTouched;
  DefaultValueAccessor(this.cd, this.renderer, this.elementRef) {
    this.onChange = (_) {};
    this.onTouched = (_) {};
    cd.valueAccessor = this;
  }
  writeValue(value) {
    // both this.value and setProperty are required at the moment

    // remove when a proper imperative API is provided
    this.value = isBlank(value) ? "" : value;
    setProperty(this.renderer, this.elementRef, "value", this.value);
  }
  void registerOnChange(fn) {
    this.onChange = fn;
  }
  void registerOnTouched(fn) {
    this.onTouched = fn;
  }
}
