library angular2.src.forms.directives.ng_model;

import "package:angular2/src/facade/async.dart"
    show EventEmitter, ObservableWrapper;
import "package:angular2/src/facade/collection.dart" show StringMapWrapper;
import "package:angular2/angular2.dart" show Directive, Ancestor, onChange;
import "package:angular2/di.dart" show Binding;
import "ng_control.dart" show NgControl;
import "../model.dart" show Control;
import "shared.dart" show setUpControl;

const formControlBinding = const Binding(NgControl, toAlias: NgModel);
/**
 * Binds a domain model to the form.
 *
 * # Example
 *  ```
 * @Component({selector: "search-comp"})
 * @View({
 *      directives: [formDirectives],
 *      template: `
              <input type='text' [(ng-model)]="searchQuery">
 *      `})
 * class SearchComp {
 *  searchQuery: string;
 * }
 *  ```
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "[ng-model]:not([ng-control]):not([ng-form-control])",
    hostInjector: const [formControlBinding],
    properties: const ["model: ng-model"],
    events: const ["ngModel"],
    lifecycle: const [onChange],
    exportAs: "form")
class NgModel extends NgControl {
  var _control = new Control("");
  var _added = false;
  var ngModel = new EventEmitter();
  dynamic model;
  onChange(c) {
    if (!this._added) {
      setUpControl(this._control, this);
      this.control.updateValidity();
      this._added = true;
    }
    ;
    if (StringMapWrapper.contains(c, "model")) {
      this._control.updateValue(this.model);
    }
  }
  get control {
    return this._control;
  }
  List<String> get path {
    return [];
  }
  void viewToModelUpdate(dynamic newValue) {
    ObservableWrapper.callNext(this.ngModel, newValue);
  }
}
