library angular2.src.forms.directives.select_control_value_accessor;

import "package:angular2/angular2.dart"
    show Directive, Query, QueryList, Renderer, ElementRef;
import "ng_control.dart" show NgControl;
import "control_value_accessor.dart" show ControlValueAccessor;
import "shared.dart" show setProperty;

/**
 * Marks <option> as dynamic, so Angular can be notified when options change.
 *
 * #Example:
 * ```
 * <select ng-control="city">
 *   <option *ng-for="#c of cities" [value]="c"></option>
 * </select>
 * ``
 * @exportedAs angular2/forms
 */
@Directive(selector: "option")
class NgSelectOption {}
/**
 * The accessor for writing a value and listening to changes on a select element.
 *
 * @exportedAs angular2/forms
 */
@Directive(
    selector: "select[ng-control],select[ng-form-control],select[ng-model]",
    host: const {
  "(change)": "onChange(\$event.target.value)",
  "(input)": "onChange(\$event.target.value)",
  "(blur)": "onTouched()",
  "[value]": "value",
  "[class.ng-untouched]": "cd.control?.untouched == true",
  "[class.ng-touched]": "cd.control?.touched == true",
  "[class.ng-pristine]": "cd.control?.pristine == true",
  "[class.ng-dirty]": "cd.control?.dirty == true",
  "[class.ng-valid]": "cd.control?.valid == true",
  "[class.ng-invalid]": "cd.control?.valid == false"
})
class SelectControlValueAccessor implements ControlValueAccessor {
  NgControl cd;
  Renderer renderer;
  ElementRef elementRef;
  var value = "";
  Function onChange;
  Function onTouched;
  SelectControlValueAccessor(this.cd, this.renderer, this.elementRef, @Query(
      NgSelectOption, descendants: true) QueryList<NgSelectOption> query) {
    this.onChange = (_) {};
    this.onTouched = (_) {};
    cd.valueAccessor = this;
    this._updateValueWhenListOfOptionsChanges(query);
  }
  writeValue(value) {
    // both this.value and setProperty are required at the moment

    // remove when a proper imperative API is provided
    this.value = value;
    setProperty(this.renderer, this.elementRef, "value", value);
  }
  void registerOnChange(fn) {
    this.onChange = fn;
  }
  void registerOnTouched(fn) {
    this.onTouched = fn;
  }
  _updateValueWhenListOfOptionsChanges(QueryList<NgSelectOption> query) {
    query.onChange(() => this.writeValue(this.value));
  }
}
