library angular2.src.render.dom.compiler.template_loader;

import "package:angular2/di.dart" show Injectable;
import "package:angular2/src/facade/lang.dart"
    show isBlank, isPresent, BaseException, stringify;
import "package:angular2/src/facade/collection.dart"
    show Map, MapWrapper, ListWrapper;
import "package:angular2/src/facade/async.dart" show PromiseWrapper, Future;
import "package:angular2/src/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/render/xhr.dart" show XHR;
import "../../api.dart" show ViewDefinition;
import "package:angular2/src/services/url_resolver.dart" show UrlResolver;

/**
 * Strategy to load component templates.
 * TODO: Make public API once we are more confident in this approach.
 */
@Injectable()
class TemplateLoader {
  XHR _xhr;
  Map<String, Future<String>> _cache = MapWrapper.create();
  TemplateLoader(this._xhr, UrlResolver urlResolver) {}
  Future<dynamic> load(ViewDefinition view) {
    var html;
    var fetchedStyles;
    // Load the HTML
    if (isPresent(view.template)) {
      html = PromiseWrapper.resolve(view.template);
    } else if (isPresent(view.templateAbsUrl)) {
      html = this._loadText(view.templateAbsUrl);
    } else {
      throw new BaseException(
          "View should have either the templateUrl or template property set");
    }
    // Load the styles
    if (isPresent(view.styleAbsUrls) && view.styleAbsUrls.length > 0) {
      fetchedStyles =
          ListWrapper.map(view.styleAbsUrls, (url) => this._loadText(url));
    } else {
      fetchedStyles = [];
    }
    // Inline the styles and return a template element
    return PromiseWrapper
        .all(ListWrapper.concat([html], fetchedStyles))
        .then((List<String> res) {
      var html = res[0];
      var fetchedStyles = ListWrapper.slice(res, 1);
      html = _createStyleTags(view.styles) +
          _createStyleTags(fetchedStyles) +
          html;
      return DOM.createTemplate(html);
    });
  }
  Future<String> _loadText(String url) {
    var response = MapWrapper.get(this._cache, url);
    if (isBlank(response)) {
      // TODO(vicb): change error when TS gets fixed

      // https://github.com/angular/angular/issues/2280

      // throw new BaseException(`Failed to fetch url "${url}"`);
      response = PromiseWrapper.catchError(this._xhr.get(url),
          (_) => PromiseWrapper.reject(
              new BaseException('''Failed to fetch url "${ url}"'''), null));
      MapWrapper.set(this._cache, url, response);
    }
    return response;
  }
}
String _createStyleTags([List<String> styles]) {
  return isBlank(styles)
      ? ""
      : ListWrapper
          .map(styles, (css) => '''<style type=\'text/css\'>${ css}</style>''')
          .join("");
}
