library angular2.src.render.dom.view.view;

import "package:angular2/src/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/facade/collection.dart"
    show ListWrapper, MapWrapper, Map, StringMapWrapper, List;
import "package:angular2/change_detection.dart" show Locals;
import "package:angular2/src/facade/lang.dart"
    show isPresent, isBlank, BaseException;
import "proto_view.dart" show DomProtoView;
import "../shadow_dom/light_dom.dart" show LightDom;
import "element.dart" show DomElement;
import "../../api.dart" show RenderViewRef, EventDispatcher;

resolveInternalDomView(RenderViewRef viewRef) {
  return ((viewRef as DomViewRef))._view;
}
class DomViewRef extends RenderViewRef {
  DomView _view;
  DomViewRef(this._view) : super() {
    /* super call moved to initializer */;
  }
}
/**
 * Const of making objects: http://jsperf.com/instantiate-size-of-object
 */
class DomView {
  DomProtoView proto;
  List<dynamic> rootNodes;
  List<dynamic> boundTextNodes;
  List<DomElement> boundElements;
  LightDom hostLightDom = null;
  var shadowRoot = null;
  bool hydrated = false;
  EventDispatcher eventDispatcher = null;
  List<Function> eventHandlerRemovers = [];
  DomView(this.proto, this.rootNodes, this.boundTextNodes, this.boundElements) {
  }
  DomElement getDirectParentElement(num boundElementIndex) {
    var binder = this.proto.elementBinders[boundElementIndex];
    var parent = null;
    if (!identical(binder.parentIndex, -1) &&
        identical(binder.distanceToParent, 1)) {
      parent = this.boundElements[binder.parentIndex];
    }
    return parent;
  }
  setElementProperty(num elementIndex, String propertyName, dynamic value) {
    var setter = MapWrapper.get(
        this.proto.elementBinders[elementIndex].propertySetters, propertyName);
    setter(this.boundElements[elementIndex].element, value);
  }
  callAction(num elementIndex, String actionExpression, dynamic actionArgs) {
    var binder = this.proto.elementBinders[elementIndex];
    var hostAction = MapWrapper.get(binder.hostActions, actionExpression);
    hostAction.eval(this.boundElements[elementIndex].element,
        this._localsWithAction(actionArgs));
  }
  Locals _localsWithAction(Object action) {
    var map = MapWrapper.create();
    MapWrapper.set(map, "\$action", action);
    return new Locals(null, map);
  }
  setText(num textIndex, String value) {
    DOM.setText(this.boundTextNodes[textIndex], value);
  }
  bool dispatchEvent(elementIndex, eventName, event) {
    var allowDefaultBehavior = true;
    if (isPresent(this.eventDispatcher)) {
      var evalLocals = MapWrapper.create();
      MapWrapper.set(evalLocals, "\$event", event);
      // TODO(tbosch): reenable this when we are parsing element properties

      // out of action expressions

      // var localValues = this.proto.elementBinders[elementIndex].eventLocals.eval(null, new

      // Locals(null, evalLocals));

      // this.eventDispatcher.dispatchEvent(elementIndex, eventName, localValues);
      allowDefaultBehavior = this.eventDispatcher.dispatchEvent(
          elementIndex, eventName, evalLocals);
      if (!allowDefaultBehavior) {
        event.preventDefault();
      }
    }
    return allowDefaultBehavior;
  }
}
